"use strict";

var _reactHooks = require("@testing-library/react-hooks");

var _useDirtyFieldValidation = require("../useDirtyFieldValidation");

var _pureValidations = require("@totalsoft/pure-validations");

describe("UseDirtyFieldValidaiton hook", () => {
  afterEach(() => {
    (0, _pureValidations.__clearMocks)();
  });
  it("should minimize the number of renders", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {
      field: "test"
    };
    let renderNo = 0;

    function renderCallack() {
      renderNo = renderNo + 1;
      return (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator);
    } // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(renderCallack);
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    });
    rerender(); // Assert

    expect(renderNo).toBe(3);
  });
  it("should validate dirty fields", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {
      field: ""
    };
    const dirtyInfo = {
      field: true
    }; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model, dirtyInfo);
    }); // Assert

    expect(_pureValidations.validate.mock.calls[0][2].dirtyInfo).toBe(dirtyInfo);
  });
  it("should validate all fields", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    }); // Assert

    expect(_pureValidations.validate.mock.calls[0][2].dirtyInfo).toBe(undefined);
  });
  it("should not use dirty info after full validation", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {};
    const dirtyInfo = {
      field: true
    }; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    });
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model, dirtyInfo);
    }); // Assert

    expect(_pureValidations.validate.mock.calls[0][2].dirtyInfo).toBe(undefined);
    expect(_pureValidations.validate.mock.calls[1][2].dirtyInfo).toBe(undefined);
  });
  it("should use dirty info after reset", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {};
    const dirtyInfo = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    });
    (0, _reactHooks.act)(() => {
      const [,, reset] = result.current;
      reset();
    });
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model, dirtyInfo);
    }); // Assert

    expect(_pureValidations.validate.mock.calls[0][2].dirtyInfo).toBe(undefined);
    expect(_pureValidations.validate.mock.calls[1][2].dirtyInfo).toBe(dirtyInfo);
  });
  it("returns success when validation result is success", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {};
    const dirtyInfo = {};
    let returnedIsValid = undefined; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      returnedIsValid = validate(model, dirtyInfo);
    }); // Assert

    const [validation] = result.current;
    expect(_pureValidations.isValid.mock.calls.length).toBe(1);
    expect(_pureValidations.validate.mock.calls.length).toBe(1);
    expect((0, _pureValidations.isValid)(validation)).toBe(true);
    expect(returnedIsValid).toBe(true);
  });
  it("returns failure when validation result is failure", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Failure(["baa"]));

    const model = {};
    let returnedIsValid = undefined;
    const dirtyInfo = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      returnedIsValid = validate(model, dirtyInfo);
    }); // Assert

    const [validation] = result.current;
    expect(_pureValidations.isValid.mock.calls.length).toBe(1);
    expect(_pureValidations.validate.mock.calls.length).toBe(1);
    expect((0, _pureValidations.isValid)(validation)).toBe(false);
    expect(returnedIsValid).toBe(false);
  });
  it("does not change the returned object references on re-render", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success()); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(() => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator, {}));
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the validator changes", () => {
    // Arrange
    const callback = () => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(_pureValidations.Validator.of(_pureValidations.Validation.Success())); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the mode (full, dirtyOnly) changes", () => {
    // Arrange
    const model = {};

    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const callback = () => (0, _useDirtyFieldValidation.useDirtyFieldValidation)(validator); // Act


    const {
      result
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    (0, _reactHooks.act)(() => {
      validate1(model);
    });
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toStrictEqual(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
    expect(_pureValidations.validate.mock.calls.length).toBe(1);
  });
});