"use strict";

var _reactHooks = require("@testing-library/react-hooks");

var _useValidation = require("../useValidation");

var _pureValidations = require("@totalsoft/pure-validations");

describe("useValidation hook", () => {
  afterEach(() => {
    (0, _pureValidations.__clearMocks)();
  });
  it("returns success when validation result is success", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {};
    let returnedIsValid = undefined; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      returnedIsValid = validate(model);
    }); // Assert

    const [validation] = result.current;
    expect(_pureValidations.isValid.mock.calls.length).toBe(1);
    expect(_pureValidations.validate.mock.calls.length).toBe(1);
    expect((0, _pureValidations.isValid)(validation)).toBe(true);
    expect(returnedIsValid).toBe(true);
  });
  it("returns failure when validation result is failure", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Failure(["baa"]));

    const model = {};
    let returnedIsValid = undefined; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      returnedIsValid = validate(model);
    }); // Assert

    const [validation] = result.current;
    expect(_pureValidations.isValid.mock.calls.length).toBe(1);
    expect(_pureValidations.validate.mock.calls.length).toBe(1);
    expect((0, _pureValidations.isValid)(validation)).toBe(false);
    expect(returnedIsValid).toBe(false);
  });
  it("returns success when validation is not run", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Failure()); // Act


    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator)); // Assert

    const [validation] = result.current;
    expect((0, _pureValidations.isValid)(validation)).toBe(true);
  });
  it("returns success after reset", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Failure());

    const model = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    });
    const [validation1] = result.current;
    (0, _reactHooks.act)(() => {
      const [,, reset] = result.current;
      reset();
    }); // Assert

    const [validation2] = result.current;
    expect((0, _pureValidations.isValid)(validation1)).toBe(false);
    expect((0, _pureValidations.isValid)(validation2)).toBe(true);
  });
  it("should minimize the number of renders", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const model = {};
    let renderNo = 0;

    function renderCallack() {
      renderNo = renderNo + 1;
      return (0, _useValidation.useValidation)(validator);
    } // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(renderCallack);
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    });
    rerender(); // Assert

    expect(renderNo).toBe(3);
  });
  it("calls the logTo HOV function once", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const logger = {
      log: _ => {}
    };
    const model = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator, {
      isLogEnabled: true,
      logger
    }));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    }); // Assert

    expect(_pureValidations.logTo.mock.calls).toEqual([[logger]]);
  });
  it("calls filterFields HOV function once", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const filterFn = _ => true;

    const model = {}; // Act

    const {
      result
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator, {
      fieldFilterFunc: filterFn
    }));
    (0, _reactHooks.act)(() => {
      const [, validate] = result.current;
      validate(model);
    }); // Assert

    expect(_pureValidations.filterFields.mock.calls).toEqual([[filterFn]]);
  });
  it("does not change the returned object references on re-render", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success()); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(() => (0, _useValidation.useValidation)(validator, {}));
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the validator changes", () => {
    // Arrange
    const callback = () => (0, _useValidation.useValidation)(_pureValidations.Validator.of(_pureValidations.Validation.Success()), {
      logger: {}
    }); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the logger changes", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const callback = () => (0, _useValidation.useValidation)(validator, {
      logger: {}
    }); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the logging flag changes", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    let isLogEnabled = true;

    function callback() {
      isLogEnabled = !isLogEnabled;
      return (0, _useValidation.useValidation)(validator, {
        isLogEnabled: !isLogEnabled
      });
    } // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when the filter fuction changes", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    const callback = () => (0, _useValidation.useValidation)(validator, {
      fieldFilterFunc: () => {}
    }); // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
  it("changes the validation function when other deps change", () => {
    // Arrange
    const validator = _pureValidations.Validator.of(_pureValidations.Validation.Success());

    let depFlag = true;

    function callback() {
      depFlag = !depFlag;
      return (0, _useValidation.useValidation)(validator, {}, [depFlag]);
    } // Act


    const {
      result,
      rerender
    } = (0, _reactHooks.renderHook)(callback);
    const [validation1, validate1, reset1] = result.current;
    rerender();
    const [validation2, validate2, reset2] = result.current; // Assert

    expect(validation1).toBe(validation2);
    expect(validate1).not.toBe(validate2);
    expect(reset1).toBe(reset2);
  });
});